/*

This program was written by Yuki Mio.
(http://www.ticalc.org/archives/files/authors/70/7071.html)

[[ You can use this source code for any purpose without permission! ]]

GTK+1.2:
gcc -Wall -g main.c -o ocfe `gtk-config --cflags --libs`
gcc -Wall -g main.c -o ocfe `pkg-config --cflags --libs gtk+`

GTK+2.0:
gcc -Wall -g main.c -o ocfe `pkg-config --cflags --libs gtk+-2.0`

*/



/*--[ Header ]----------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdktypes.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>



/*--[ Define ]----------------------------------------------------------------*/
#define OCFE_APPNAME        "Omnicalc Font Editor for GTK+"
#define OCFE_VERSION        "1.01"
#define OCFE_TITLE OCFE_APPNAME " v" OCFE_VERSION
#define OCFE_HELP           "This program supports to make Omnicalc Font File."
#define OCFE_AUTHOR_ADDRESS "miotti@plum.ocn.ne.jp\n" \
                            "http://www.ticalc.org/archives/files/authors/70/7071.html\n" \
                            "http://ticalc.fc2web.com/"
#define OCFE_AUTHOR_NAME    "This program was written by Yuki Mio."
#define CONFIRMATION_TITLE  "Confirmation"
#define CONFIRMATION_MSG    "This operation cannot be undone.\n" \
                            "Are you sure you want to proceed?"
#define ERROR_COULDNOT_OPEN "ERROR:Couldn't open file.\n"



/*--[ Icon ]------------------------------------------------------------------*/
static char *icon[] = {
"32 32 2 1",
"  c #CCE0CF",
"@ c #312E22",
"  @@@                @          ",
" @   @                        @ ",
" @   @ @@ @  @ @@   @@          ",
" @   @ @ @ @ @@  @   @        @ ",
" @   @ @ @ @ @   @   @          ",
" @   @ @   @ @   @   @        @ ",
"  @@@  @   @ @   @  @@@         ",
"                              @ ",
"  @@@         @@                ",
" @   @         @                ",
" @      @@@    @    @@@         ",
" @         @   @   @            ",
" @      @@@@   @   @            ",
" @   @ @   @   @   @   @        ",
"  @@@   @@@@  @@@   @@@         ",
"                                ",
" @@@@@              @           ",
" @                  @           ",
" @      @@@  @ @@  @@@          ",
" @@@@  @   @ @@  @  @           ",
" @     @   @ @   @  @           ",
" @     @   @ @   @  @  @        ",
" @      @@@  @   @   @@         ",
"                                ",
" @@@@@     @  @  @              ",
" @         @     @              ",
" @      @@ @ @@ @@@   @@@  @ @@ ",
" @@@@  @  @@  @  @   @   @ @@  @",
" @     @   @  @  @   @   @ @    ",
" @     @   @  @  @  @@   @ @    ",
" @@@@@  @@@@ @@@  @@  @@@  @    ",
"                                "
};



/*--[ Description of Characters ]---------------------------------------------*/
  char *cDesc[] = 
  {
    "(None)", /* 000 */
    "italic \"n\"",
    "symbol \"u\"",
    "symbol \"v\"",
    "symbol \"w\"",
    "rightward arrow(cursor)",
    "upward arrow(short)",
    "downward arrow(short)",
    "integral",
    "times",
    "box icon", /* 010 */
    "cross icon",
    "dot icon",
    "small \"T\"",
    "3rd power symbol",
    "bold \"F\"",
    "route",
    "inverse power symbol",
    "2nd power symbol",
    "angle",
    "degree", /* 020 */
    "radian",
    "transpose",
    "<=",
    "not equal",
    ">=",
    "-(sign)",
    "exponential",
    "rightward arrow(store)",
    "small \"10\"",
    "upward arrow(thin)", /* 030 */
    "downward arrow(thin)",
    "(space)",
    "!",
    "\"",
    "#",
    "4th power symbol",
    "%",
    "&",
    "\'",
    "(",
    ")",
    "*",
    "+",
    ",(comma)",
    "-(operator)",
    ".(dot)",
    "/",
    "0","1","2","3","4","5","6","7","8","9", /* 048-057 */
    ":",";","<","=",">","?","@",             /* 058-064 */
    "A","B","C","D","E","F","G","H","I","J", /* 065-074 */
    "K","L","M","N","O","P","Q","R","S","T", /* 075-084 */
    "U","V","W","X","Y","Z",                 /* 085-090 */
    "theta","\\","]","^","_","`",            /* 091-096 */
    "a","b","c","d","e","f","g","h","i","j", /* 097-106 */
    "k","l","m","n","o","p","q","r","s","t", /* 107-116 */
    "u","v","w","x","y","z",                 /* 117-122 */
    "{",
    "|",
    "}",
    "~",
    "inversed equal",
    "small \"0\"",
    "small \"1\"",
    "small \"2\"",
    "small \"3\"",
    "small \"4\"",
    "small \"5\"",
    "small \"6\"",
    "small \"7\"",
    "small \"8\"",
    "small \"9\"", /* 133-137 */
    "\"A\" with acute accent",
    "\"A\" with grave accent",
    "\"A\" with circumflex", /* 140 */
    "\"A\" with diaeresis",
    "\"a\" with acute accent",
    "\"a\" with grave accent",
    "\"a\" with circumflex",
    "\"a\" with diaeresis",
    "\"E\" with acute accent",
    "\"E\" with grave accent",
    "\"E\" with circumflex",
    "\"E\" with diaeresis",
    "\"e\" with acute accent", /* 150 */
    "\"e\" with grave accent",
    "\"e\" with circumflex",
    "\"e\" with diaeresis",
    "\"I\" with acute accent",
    "\"I\" with grave accent",
    "\"I\" with circumflex",
    "\"I\" with diaeresis",
    "\"i\" with acute accent",
    "\"i\" with grave accent",
    "\"i\" with circumflex", /* 160 */
    "\"i\" with diaeresis",
    "\"O\" with acute accent",
    "\"O\" with grave accent",
    "\"O\" with circumflex",
    "\"O\" with diaeresis",
    "\"o\" with acute accent",
    "\"o\" with grave accent",
    "\"o\" with circumflex",
    "\"o\" with diaeresis",
    "\"U\" with acute accent", /* 170 */
    "\"U\" with grave accent",
    "\"U\" with circumflex",
    "\"U\" with diaeresis",
    "\"u\" with acute accent",
    "\"u\" with grave accent",
    "\"u\" with circumflex",
    "\"u\" with diaeresis",
    "\"C\" with cedilla",
    "\"c\" with cedilla",
    "tilde over \"N\"", /* 180 */
    "tilde over \"n\"",
    "acute accent",
    "acute accent",
    "diaeresis",
    "upside-down \"?\"",
    "upside-down \"!\"",
    "Alpha(Upper case)",
    "Beta(Upper case)",
    "Gamma(Upper case)",
    "Delta(Upper case)", /* 190 */
    "Delta(Lower case)",
    "Epsilon(Lower case)",
    "[",
    "Lambda(Lower case)",
    "Mu(Lower case)",
    "Pi(Lower case)",
    "Rho(Lower case)",
    "Sigma(Upper case)",
    "Sigma(Lower case)",
    "Tau(Lower case)", /* 200 */
    "Phi(Lower case)",
    "Omega(Upper case)",
    "\"X\" with upperline",
    "\"Y\" with upperline",
    "X th power symbol",
    "...(ellipsis)",
    "leftward arrow(cursor)",
    "block",
    "per",
    "hyphen", /* 210 */
    "area",
    "temperature",
    "cube",
    "(Enter)",
    "imaginary number sign",
    "\"P\" with circumflex",
    "chi(Lowercase)",
    "F",
    "natural logarithm",
    "small \"L\"(List)", /* 220 */
    "bold \"N\"",
    "))",
    "rightward arrow(short)",
    "cursor(Text mode)",
    "\"2nd\" lock",
    "\"ALPHA\" lock",
    "\"alpha\" lock",
    "cursor(Text insert)",
    "\"2nd\" & insert lock",
    "\"ALPHA\" & insert lock", /* 230 */
    "\"alpha\" & insert lock",
    "thin line",
    "bold line",
    "fill above",
    "fill below",
    "animation & line",
    "animation",
    "dot line",
    "upward arrow(long)",
    "downward arrow(long)", /* 240 */
    "checked pattern",
    "$",
    "upward arrow2(short)",
    "Eszett",
    "(None)", "(None)", "(None)", "(None)", "(None)", "(None)", /* 250 */
    "(None)", "(None)", "(None)", "(None)", "(None)" /* 255 */
  };


/*--[ Default font data ]-----------------------------------------------------*/
unsigned char font_data[] = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 000 */
0x00, 0x00, 0x16, 0x09, 0x09, 0x12, 0x12, /* 001 */
0x00, 0x00, 0x11, 0x11, 0x11, 0x13, 0x0D, /* 002 */
0x00, 0x00, 0x11, 0x11, 0x0A, 0x0A, 0x04, /* 003 */
0x00, 0x00, 0x11, 0x11, 0x15, 0x15, 0x0A, /* 004 */
0x00, 0x08, 0x0C, 0x0E, 0x0C, 0x08, 0x00, /* 005 */
0x04, 0x0E, 0x1F, 0x0E, 0x0E, 0x00, 0x00, /* 006 */
0x00, 0x00, 0x0E, 0x0E, 0x1F, 0x0E, 0x04, /* 007 */
0x02, 0x05, 0x04, 0x04, 0x04, 0x14, 0x08, /* 008 */
0x00, 0x11, 0x0A, 0x04, 0x0A, 0x11, 0x00, /* 009 */
0x00, 0x00, 0x00, 0x0E, 0x0A, 0x0E, 0x00, /* 010 */
0x00, 0x00, 0x00, 0x04, 0x0E, 0x04, 0x00, /* 011 */
0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, /* 012 */
0x00, 0x00, 0x00, 0x1C, 0x08, 0x08, 0x08, /* 013 */
0x0C, 0x02, 0x0C, 0x02, 0x0C, 0x00, 0x00, /* 014 */
0x1F, 0x18, 0x18, 0x1E, 0x18, 0x18, 0x18, /* 015 */
0x07, 0x04, 0x04, 0x04, 0x14, 0x0C, 0x04, /* 016 */
0x03, 0x01, 0x19, 0x01, 0x01, 0x00, 0x00, /* 017 */
0x0C, 0x02, 0x04, 0x08, 0x0E, 0x00, 0x00, /* 018 */
0x00, 0x00, 0x01, 0x02, 0x04, 0x08, 0x1F, /* 019 */
0x0C, 0x12, 0x12, 0x0C, 0x00, 0x00, 0x00, /* 020 */
0x16, 0x18, 0x10, 0x10, 0x00, 0x00, 0x00, /* 021 */
0x0E, 0x04, 0x04, 0x04, 0x04, 0x00, 0x00, /* 022 */
0x01, 0x06, 0x18, 0x06, 0x01, 0x00, 0x1F, /* 023 */
0x02, 0x02, 0x1F, 0x04, 0x1F, 0x08, 0x08, /* 024 */
0x10, 0x0C, 0x03, 0x0C, 0x10, 0x00, 0x1F, /* 025 */
0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, /* 026 */
0x00, 0x00, 0x0F, 0x08, 0x0E, 0x08, 0x0F, /* 027 */
0x00, 0x04, 0x02, 0x1F, 0x02, 0x04, 0x00, /* 028 */
0x00, 0x00, 0x17, 0x15, 0x15, 0x15, 0x17, /* 029 */
0x04, 0x0E, 0x15, 0x04, 0x04, 0x04, 0x04, /* 030 */
0x04, 0x04, 0x04, 0x04, 0x15, 0x0E, 0x04, /* 031 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 032 */
0x04, 0x04, 0x04, 0x04, 0x00, 0x04, 0x04, /* 033 */
0x0A, 0x0A, 0x0A, 0x00, 0x00, 0x00, 0x00, /* 034 */
0x0A, 0x0A, 0x1F, 0x0A, 0x1F, 0x0A, 0x0A, /* 035 */
0x08, 0x0A, 0x0E, 0x02, 0x02, 0x00, 0x00, /* 036 */
0x18, 0x19, 0x02, 0x04, 0x08, 0x13, 0x03, /* 037 */
0x08, 0x14, 0x14, 0x08, 0x15, 0x12, 0x0D, /* 038 */
0x04, 0x04, 0x04, 0x00, 0x00, 0x00, 0x00, /* 039 */
0x02, 0x04, 0x08, 0x08, 0x08, 0x04, 0x02, /* 040 */
0x08, 0x04, 0x02, 0x02, 0x02, 0x04, 0x08, /* 041 */
0x00, 0x04, 0x15, 0x0E, 0x15, 0x04, 0x00, /* 042 */
0x00, 0x04, 0x04, 0x1F, 0x04, 0x04, 0x00, /* 043 */
0x00, 0x00, 0x00, 0x00, 0x0C, 0x04, 0x08, /* 044 */
0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 0x00, /* 045 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, /* 046 */
0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x00, /* 047 */
0x0E, 0x11, 0x13, 0x15, 0x19, 0x11, 0x0E, /* 048 */
0x04, 0x0C, 0x04, 0x04, 0x04, 0x04, 0x0E, /* 049 */
0x0E, 0x11, 0x01, 0x02, 0x04, 0x08, 0x1F, /* 050 */
0x1F, 0x02, 0x04, 0x02, 0x01, 0x11, 0x0E, /* 051 */
0x02, 0x06, 0x0A, 0x12, 0x1F, 0x02, 0x02, /* 052 */
0x1F, 0x10, 0x1E, 0x01, 0x01, 0x11, 0x0E, /* 053 */
0x06, 0x08, 0x10, 0x1E, 0x11, 0x11, 0x0E, /* 054 */
0x1F, 0x01, 0x02, 0x04, 0x08, 0x08, 0x08, /* 055 */
0x0E, 0x11, 0x11, 0x0E, 0x11, 0x11, 0x0E, /* 056 */
0x0E, 0x11, 0x11, 0x0F, 0x01, 0x02, 0x0C, /* 057 */
0x00, 0x0C, 0x0C, 0x00, 0x0C, 0x0C, 0x00, /* 058 */
0x00, 0x0C, 0x0C, 0x00, 0x0C, 0x04, 0x08, /* 059 */
0x02, 0x04, 0x08, 0x10, 0x08, 0x04, 0x02, /* 060 */
0x00, 0x00, 0x1F, 0x00, 0x1F, 0x00, 0x00, /* 061 */
0x08, 0x04, 0x02, 0x01, 0x02, 0x04, 0x08, /* 062 */
0x0E, 0x11, 0x01, 0x02, 0x04, 0x00, 0x04, /* 063 */
0x0E, 0x11, 0x15, 0x17, 0x14, 0x10, 0x0F, /* 064 */
0x0E, 0x11, 0x11, 0x1F, 0x11, 0x11, 0x11, /* 065 */
0x1E, 0x11, 0x11, 0x1E, 0x11, 0x11, 0x1E, /* 066 */
0x0E, 0x11, 0x10, 0x10, 0x10, 0x11, 0x0E, /* 067 */
0x1E, 0x11, 0x11, 0x11, 0x11, 0x11, 0x1E, /* 068 */
0x1F, 0x10, 0x10, 0x1E, 0x10, 0x10, 0x1F, /* 069 */
0x1F, 0x10, 0x10, 0x1E, 0x10, 0x10, 0x10, /* 070 */
0x0E, 0x11, 0x10, 0x17, 0x11, 0x11, 0x0F, /* 071 */
0x11, 0x11, 0x11, 0x1F, 0x11, 0x11, 0x11, /* 072 */
0x0E, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0E, /* 073 */
0x07, 0x02, 0x02, 0x02, 0x02, 0x12, 0x0C, /* 074 */
0x11, 0x12, 0x14, 0x18, 0x14, 0x12, 0x11, /* 075 */
0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x1F, /* 076 */
0x11, 0x1B, 0x15, 0x15, 0x11, 0x11, 0x11, /* 077 */
0x11, 0x11, 0x19, 0x15, 0x13, 0x11, 0x11, /* 078 */
0x0E, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 079 */
0x1E, 0x11, 0x11, 0x1E, 0x10, 0x10, 0x10, /* 080 */
0x0E, 0x11, 0x11, 0x11, 0x15, 0x12, 0x0D, /* 081 */
0x1E, 0x11, 0x11, 0x1E, 0x14, 0x12, 0x11, /* 082 */
0x0F, 0x10, 0x10, 0x0E, 0x01, 0x01, 0x1E, /* 083 */
0x1F, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, /* 084 */
0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 085 */
0x11, 0x11, 0x11, 0x11, 0x0A, 0x0A, 0x04, /* 086 */
0x11, 0x11, 0x11, 0x11, 0x15, 0x15, 0x0A, /* 087 */
0x11, 0x11, 0x0A, 0x04, 0x0A, 0x11, 0x11, /* 088 */
0x11, 0x11, 0x11, 0x0A, 0x04, 0x04, 0x04, /* 089 */
0x1F, 0x01, 0x02, 0x04, 0x08, 0x10, 0x1F, /* 090 */
0x06, 0x09, 0x11, 0x1F, 0x11, 0x12, 0x0C, /* 091 */
0x00, 0x10, 0x08, 0x04, 0x02, 0x01, 0x00, /* 092 */
0x0C, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0C, /* 093 */
0x04, 0x0A, 0x11, 0x00, 0x00, 0x00, 0x00, /* 094 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, /* 095 */
0x04, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00, /* 096 */
0x00, 0x00, 0x0E, 0x01, 0x0F, 0x11, 0x0F, /* 097 */
0x10, 0x10, 0x16, 0x19, 0x11, 0x11, 0x1E, /* 098 */
0x00, 0x00, 0x0E, 0x10, 0x10, 0x11, 0x0E, /* 099 */
0x01, 0x01, 0x0D, 0x13, 0x11, 0x11, 0x0F, /* 100 */
0x00, 0x00, 0x0E, 0x11, 0x1F, 0x10, 0x0E, /* 101 */
0x06, 0x09, 0x08, 0x1C, 0x08, 0x08, 0x08, /* 102 */
0x00, 0x0F, 0x11, 0x11, 0x0F, 0x01, 0x0E, /* 103 */
0x10, 0x10, 0x16, 0x19, 0x11, 0x11, 0x11, /* 104 */
0x04, 0x00, 0x0C, 0x04, 0x04, 0x04, 0x0E, /* 105 */
0x02, 0x00, 0x06, 0x02, 0x02, 0x12, 0x0C, /* 106 */
0x08, 0x08, 0x09, 0x0A, 0x0C, 0x0A, 0x09, /* 107 */
0x0C, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0E, /* 108 */
0x00, 0x00, 0x1A, 0x15, 0x15, 0x11, 0x11, /* 109 */
0x00, 0x00, 0x16, 0x19, 0x11, 0x11, 0x11, /* 110 */
0x00, 0x00, 0x0E, 0x11, 0x11, 0x11, 0x0E, /* 111 */
0x00, 0x00, 0x1E, 0x11, 0x1E, 0x10, 0x10, /* 112 */
0x00, 0x00, 0x0D, 0x13, 0x0F, 0x01, 0x01, /* 113 */
0x00, 0x00, 0x16, 0x19, 0x10, 0x10, 0x10, /* 114 */
0x00, 0x00, 0x0E, 0x10, 0x0E, 0x01, 0x1E, /* 115 */
0x08, 0x08, 0x1C, 0x08, 0x08, 0x09, 0x06, /* 116 */
0x00, 0x00, 0x11, 0x11, 0x11, 0x13, 0x0D, /* 117 */
0x00, 0x00, 0x11, 0x11, 0x11, 0x0A, 0x04, /* 118 */
0x00, 0x00, 0x11, 0x11, 0x15, 0x15, 0x0A, /* 119 */
0x00, 0x00, 0x11, 0x0A, 0x04, 0x0A, 0x11, /* 120 */
0x00, 0x00, 0x11, 0x11, 0x0F, 0x01, 0x0E, /* 121 */
0x00, 0x00, 0x1F, 0x02, 0x04, 0x08, 0x1F, /* 122 */
0x03, 0x04, 0x04, 0x08, 0x04, 0x04, 0x03, /* 123 */
0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, /* 124 */
0x18, 0x04, 0x04, 0x02, 0x04, 0x04, 0x18, /* 125 */
0x00, 0x08, 0x15, 0x02, 0x00, 0x00, 0x00, /* 126 */
0x1F, 0x1F, 0x11, 0x1F, 0x11, 0x1F, 0x1F, /* 127 */
0x00, 0x00, 0x0E, 0x0A, 0x0A, 0x0A, 0x0E, /* 128 */
0x00, 0x00, 0x04, 0x0C, 0x04, 0x04, 0x04, /* 129 */
0x00, 0x00, 0x0C, 0x02, 0x04, 0x08, 0x0E, /* 130 */
0x00, 0x00, 0x0C, 0x02, 0x04, 0x02, 0x0C, /* 131 */
0x00, 0x00, 0x08, 0x0A, 0x0E, 0x02, 0x02, /* 132 */
0x00, 0x00, 0x0E, 0x08, 0x0C, 0x02, 0x0C, /* 133 */
0x00, 0x00, 0x06, 0x08, 0x0E, 0x0A, 0x0E, /* 134 */
0x00, 0x00, 0x0E, 0x02, 0x04, 0x08, 0x08, /* 135 */
0x00, 0x00, 0x0E, 0x0A, 0x0E, 0x0A, 0x0E, /* 136 */
0x00, 0x00, 0x0E, 0x0A, 0x0E, 0x02, 0x0C, /* 137 */
0x02, 0x04, 0x0E, 0x11, 0x11, 0x1F, 0x11, /* 138 */
0x08, 0x04, 0x0E, 0x11, 0x11, 0x1F, 0x11, /* 139 */
0x04, 0x0A, 0x00, 0x0E, 0x11, 0x1F, 0x11, /* 140 */
0x0A, 0x00, 0x0E, 0x11, 0x11, 0x1F, 0x11, /* 141 */
0x02, 0x04, 0x0E, 0x01, 0x0F, 0x11, 0x0F, /* 142 */
0x08, 0x04, 0x0E, 0x01, 0x0F, 0x11, 0x0F, /* 143 */
0x04, 0x0A, 0x0E, 0x01, 0x0F, 0x11, 0x0F, /* 144 */
0x0A, 0x00, 0x0E, 0x01, 0x0F, 0x11, 0x0F, /* 145 */
0x02, 0x04, 0x1F, 0x10, 0x1E, 0x10, 0x1F, /* 146 */
0x08, 0x04, 0x1F, 0x10, 0x1E, 0x10, 0x1F, /* 147 */
0x04, 0x0A, 0x1F, 0x10, 0x1E, 0x10, 0x1F, /* 148 */
0x0A, 0x00, 0x1F, 0x10, 0x1E, 0x10, 0x1F, /* 149 */
0x02, 0x04, 0x0E, 0x11, 0x1F, 0x10, 0x0E, /* 150 */
0x08, 0x04, 0x0E, 0x11, 0x1F, 0x10, 0x0E, /* 151 */
0x04, 0x0A, 0x0E, 0x11, 0x1F, 0x10, 0x0E, /* 152 */
0x0A, 0x00, 0x0E, 0x11, 0x1F, 0x10, 0x0E, /* 153 */
0x02, 0x04, 0x0E, 0x04, 0x04, 0x04, 0x0E, /* 154 */
0x08, 0x04, 0x0E, 0x04, 0x04, 0x04, 0x0E, /* 155 */
0x04, 0x0A, 0x0E, 0x04, 0x04, 0x04, 0x0E, /* 156 */
0x0A, 0x00, 0x0E, 0x04, 0x04, 0x04, 0x0E, /* 157 */
0x02, 0x04, 0x00, 0x0C, 0x04, 0x04, 0x0E, /* 158 */
0x08, 0x04, 0x00, 0x0C, 0x04, 0x04, 0x0E, /* 159 */
0x04, 0x0A, 0x00, 0x0C, 0x04, 0x04, 0x0E, /* 160 */
0x0A, 0x00, 0x00, 0x0C, 0x04, 0x04, 0x0E, /* 161 */
0x02, 0x04, 0x0E, 0x11, 0x11, 0x11, 0x0E, /* 162 */
0x08, 0x04, 0x0E, 0x11, 0x11, 0x11, 0x0E, /* 163 */
0x04, 0x0A, 0x0E, 0x11, 0x11, 0x11, 0x0E, /* 164 */
0x0A, 0x00, 0x0E, 0x11, 0x11, 0x11, 0x0E, /* 165 */
0x02, 0x04, 0x00, 0x0E, 0x11, 0x11, 0x0E, /* 166 */
0x08, 0x04, 0x00, 0x0E, 0x11, 0x11, 0x0E, /* 167 */
0x04, 0x0A, 0x00, 0x0E, 0x11, 0x11, 0x0E, /* 168 */
0x0A, 0x00, 0x00, 0x0E, 0x11, 0x11, 0x0E, /* 169 */
0x02, 0x04, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 170 */
0x08, 0x04, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 171 */
0x04, 0x0A, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 172 */
0x0A, 0x00, 0x11, 0x11, 0x11, 0x11, 0x0E, /* 173 */
0x02, 0x04, 0x11, 0x11, 0x11, 0x13, 0x0D, /* 174 */
0x08, 0x04, 0x11, 0x11, 0x11, 0x13, 0x0D, /* 175 */
0x04, 0x0A, 0x00, 0x11, 0x11, 0x13, 0x0D, /* 176 */
0x0A, 0x00, 0x11, 0x11, 0x11, 0x13, 0x0D, /* 177 */
0x0E, 0x11, 0x10, 0x11, 0x0E, 0x04, 0x1C, /* 178 */
0x00, 0x00, 0x0E, 0x10, 0x11, 0x0E, 0x1C, /* 179 */
0x05, 0x0A, 0x11, 0x19, 0x15, 0x13, 0x11, /* 180 */
0x05, 0x0A, 0x00, 0x16, 0x19, 0x11, 0x11, /* 181 */
0x01, 0x02, 0x04, 0x00, 0x00, 0x00, 0x00, /* 182 */
0x10, 0x08, 0x04, 0x00, 0x00, 0x00, 0x00, /* 183 */
0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, /* 184 */
0x04, 0x00, 0x04, 0x08, 0x10, 0x11, 0x0E, /* 185 */
0x04, 0x04, 0x00, 0x04, 0x04, 0x04, 0x04, /* 186 */
0x00, 0x00, 0x0D, 0x12, 0x12, 0x12, 0x0D, /* 187 */
0x06, 0x09, 0x09, 0x0E, 0x09, 0x09, 0x16, /* 188 */
0x00, 0x00, 0x09, 0x15, 0x02, 0x02, 0x02, /* 189 */
0x00, 0x00, 0x00, 0x04, 0x0A, 0x11, 0x1F, /* 190 */
0x06, 0x08, 0x04, 0x02, 0x0F, 0x11, 0x0E, /* 191 */
0x00, 0x00, 0x0E, 0x10, 0x1E, 0x10, 0x0E, /* 192 */
0x06, 0x04, 0x04, 0x04, 0x04, 0x04, 0x06, /* 193 */
0x00, 0x08, 0x04, 0x02, 0x06, 0x09, 0x11, /* 194 */
0x00, 0x00, 0x12, 0x12, 0x12, 0x1D, 0x10, /* 195 */
0x00, 0x00, 0x1F, 0x0A, 0x0A, 0x0A, 0x13, /* 196 */
0x00, 0x06, 0x09, 0x09, 0x0E, 0x08, 0x10, /* 197 */
0x1F, 0x08, 0x04, 0x02, 0x04, 0x08, 0x1F, /* 198 */
0x00, 0x00, 0x0F, 0x12, 0x12, 0x12, 0x0C, /* 199 */
0x00, 0x00, 0x0F, 0x14, 0x04, 0x05, 0x02, /* 200 */
0x04, 0x04, 0x0E, 0x15, 0x0E, 0x04, 0x04, /* 201 */
0x0E, 0x11, 0x11, 0x11, 0x0A, 0x0A, 0x1B, /* 202 */
0x1F, 0x00, 0x11, 0x0A, 0x04, 0x0A, 0x11, /* 203 */
0x1F, 0x00, 0x11, 0x11, 0x0F, 0x01, 0x0E, /* 204 */
0x05, 0x02, 0x05, 0x00, 0x00, 0x00, 0x00, /* 205 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x15, /* 206 */
0x00, 0x02, 0x06, 0x0E, 0x06, 0x02, 0x00, /* 207 */
0x00, 0x00, 0x1C, 0x1C, 0x1C, 0x00, 0x00, /* 208 */
0x02, 0x02, 0x02, 0x04, 0x08, 0x08, 0x08, /* 209 */
0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, /* 210 */
0x0E, 0x02, 0x0E, 0x08, 0x0E, 0x00, 0x00, /* 211 */
0x0E, 0x0A, 0x0E, 0x00, 0x00, 0x00, 0x00, /* 212 */
0x0C, 0x02, 0x04, 0x02, 0x0C, 0x00, 0x00, /* 213 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 214 */
0x04, 0x00, 0x18, 0x08, 0x08, 0x0A, 0x04, /* 215 */
0x04, 0x0A, 0x1E, 0x11, 0x1E, 0x10, 0x10, /* 216 */
0x10, 0x09, 0x0A, 0x04, 0x0A, 0x12, 0x01, /* 217 */
0x1F, 0x11, 0x14, 0x1C, 0x14, 0x10, 0x10, /* 218 */
0x00, 0x00, 0x0E, 0x11, 0x1E, 0x10, 0x0C, /* 219 */
0x00, 0x00, 0x04, 0x04, 0x04, 0x04, 0x07, /* 220 */
0x11, 0x19, 0x1D, 0x1F, 0x17, 0x13, 0x11, /* 221 */
0x14, 0x0A, 0x05, 0x05, 0x05, 0x0A, 0x14, /* 222 */
0x00, 0x08, 0x1C, 0x1E, 0x1C, 0x08, 0x00, /* 223 */
0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, /* 224 */
0x1F, 0x1B, 0x11, 0x0A, 0x1B, 0x1B, 0x1B, /* 225 */
0x1F, 0x1B, 0x15, 0x11, 0x15, 0x15, 0x1F, /* 226 */
0x1F, 0x1B, 0x1D, 0x19, 0x15, 0x19, 0x1F, /* 227 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, /* 228 */
0x04, 0x0E, 0x15, 0x04, 0x04, 0x00, 0x1F, /* 229 */
0x04, 0x0A, 0x0E, 0x0A, 0x0A, 0x00, 0x1F, /* 230 */
0x04, 0x02, 0x06, 0x0A, 0x06, 0x00, 0x1F, /* 231 */
0x00, 0x00, 0x08, 0x04, 0x02, 0x01, 0x00, /* 232 */
0x18, 0x1C, 0x0E, 0x07, 0x03, 0x01, 0x00, /* 233 */
0x1F, 0x0F, 0x07, 0x03, 0x01, 0x00, 0x00, /* 234 */
0x00, 0x00, 0x10, 0x18, 0x1C, 0x1E, 0x1F, /* 235 */
0x00, 0x02, 0x05, 0x1D, 0x05, 0x02, 0x00, /* 236 */
0x00, 0x04, 0x0A, 0x0A, 0x0A, 0x04, 0x00, /* 237 */
0x00, 0x10, 0x00, 0x04, 0x00, 0x01, 0x00, /* 238 */
0x04, 0x0E, 0x1F, 0x0E, 0x0E, 0x0E, 0x00, /* 239 */
0x00, 0x0E, 0x0E, 0x0E, 0x1F, 0x0E, 0x04, /* 240 */
0x15, 0x0A, 0x15, 0x0A, 0x15, 0x0A, 0x15, /* 241 */
0x04, 0x0F, 0x14, 0x0E, 0x05, 0x1E, 0x04, /* 242 */
0x04, 0x0E, 0x1F, 0x0E, 0x0E, 0x00, 0x00, /* 243 */
0x0C, 0x12, 0x12, 0x16, 0x11, 0x11, 0x16, /* 244 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 245 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 246 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 247 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 248 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 249 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 250 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 251 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 252 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 253 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 254 */
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 255 */
};



/*--[ Color for Drawing ]-----------------------------------------------------*/
  char *ColorNames[] = 
  {
    "white",
    "black",
    "gray",
    "red",
    "green",
    "blue"
  };

  typedef enum TagColorCode
  {
    COLOR_BG,     /* white */
    COLOR_FG,     /* black */
    COLOR_BORDER, /* gray  */
    COLOR_FRAME,  /* red   */
    COLOR_ACTIVE, /* green */
    COLOR_URI,    /* blue  */
  } ColorCode;

  GdkColor Colors[6];



/*--[ Font Data ]-------------------------------------------------------------*/
  typedef struct tagTIFont
  {
    unsigned char data[5][7];
  } TIFont;

  TIFont TIFonts[256]; /* Main */
  TIFont TIFontd[256]; /* Default */

  unsigned char clipbrd[5][7]; /* Clipboard */



/*--[ Global ]----------------------------------------------------------------*/
  int activeChar = 0; /* [0-255] */
  int dotSize;        /* [2-8] */
  int mainXSize;
  int mainYSize;
  int mainWidth;
  int mainHeight;
  int dotSize2;
  int subWidth;
  int subHeight;

  char compname[9];

  GtkStyle *ver_title_style;
  GtkStyle *ver_addr_style;



/*--[ Widgets ]---------------------------------------------------------------*/
GtkWidget *window;
  GtkWidget *vb_menu;
    GtkWidget *menu;
    GtkWidget *hb_main;
      GtkWidget *eb_main;
        GdkPixmap *buf_main;
        GtkWidget *da_main;
      GtkWidget *vb_sub;
        GtkWidget *lb_desc;
        GtkWidget *eb_sub;
          GtkWidget *da_sub;
          GdkPixmap *buf_sub;

  GtkWidget *win_help;
  GtkWidget *win_ver;

  GdkPixmap *ico_pmap;
  GdkBitmap *ico_mask;



/*--[ Signal Functions ]------------------------------------------------------*/
  gboolean window_delete(GtkWidget *widget, GdkEvent *event, gpointer data);
  void window_destroy(GtkWidget *widget, gpointer gd);
  gboolean window_key_press(GtkWidget *widget, GdkEventKey *event, gpointer gd);

  gboolean da_main_expose(GtkWidget *widget, GdkEventExpose *event, gpointer gd);
  gboolean da_main_button_press(GtkWidget *widget, 
                                GdkEventButton *event, gpointer gd);
  gboolean da_sub_expose(GtkWidget *widget, GdkEventExpose *event, gpointer gd);
  gboolean da_sub_button_press(GtkWidget *widget, 
                               GdkEventButton *event, gpointer gd);

  void file_open_clicked(GtkMenuItem *item, gpointer gd);
  void file_save_clicked(GtkMenuItem *item, gpointer gd);
  void file_comp_clicked(GtkMenuItem *item, gpointer gd);
  void file_comp_clicked_2nd(GtkWidget *widget, gpointer gd);
  void file_decp_clicked(GtkMenuItem *item, gpointer gd);
  void file_img_XPM_clicked(GtkMenuItem *item, gpointer gd);
  void file_img_BMP_clicked(GtkMenuItem *item, gpointer gd);
  void file_asm_right_clicked(GtkMenuItem *item, gpointer gd);
  void file_asm_left_clicked(GtkMenuItem *item, gpointer gd);
  void file_font_BDF_clicked(GtkMenuItem *item, gpointer gd);

  void edit_size_clicked(GtkMenuItem *item, gpointer gd);
  void edit_cut_clicked(GtkMenuItem *item, gpointer gd);
  void edit_copy_clicked(GtkMenuItem *item, gpointer gd);
  void edit_paste_clicked(GtkMenuItem *item, gpointer gd);
  void edit_def_clicked(GtkMenuItem *item, gpointer gd);
  void edit_clr_clicked(GtkMenuItem *item, gpointer gd);
  void edit_inv_clicked(GtkMenuItem *item, gpointer gd);
  void edit_fh_clicked(GtkMenuItem *item, gpointer gd);
  void edit_fv_clicked(GtkMenuItem *item, gpointer gd);
  void edit_rtt_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_def_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_clr_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_inv_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_fh_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_fv_clicked(GtkMenuItem *item, gpointer gd);
  void edit_all_rtt_clicked(GtkMenuItem *item, gpointer gd);

  void help_help_clicked(GtkMenuItem *item, gpointer gd);
  void help_ver_clicked(GtkMenuItem *item, gpointer gd);



/*--[ Callback Functions ]----------------------------------------------------*/
  void redraw_main(void);
  void redraw_sub(void);
  void redraw_desc(void);
  void redraw_all(void);

  void key_press(int key, int state);
  void changeSize(int size);

  void callback_file_open_clicked(char *filename);
  void callback_file_save_clicked(char *filename);
  void callback_file_comp_clicked(char *filename);
  void callback_file_decp_clicked(char *filename);
  void callback_file_img_XPM_clicked(char *filename);
  void callback_file_img_BMP_clicked(char *filename);
  void callback_file_asm_right_clicked(char *filename);
  void callback_file_asm_left_clicked(char *filename);
  void callback_file_font_BDF_clicked(char *filename);

  void char_cut(void);
  void char_copy(void);
  void char_paste(void);
  void char_def(void);
  void char_clr(void);
  void char_inv(void);
  void char_fh(void);
  void char_fv(void);
  void char_rtt(void);
  void char_all_def(void);
  void char_all_clr(void);
  void char_all_inv(void);
  void char_all_fh(void);
  void char_all_fv(void);
  void char_all_rtt(void);



/*--[ Addtional Functions ]---------------------------------------------------*/
  void gtk_menu_item_parse_uline(GtkWidget *item);
  void gtk_label_set_uline(GtkWidget *label);



/*--[ File Dialog ]-----------------------------------------------------------*/
  GtkWidget *filesel;
  void gtk_file_dialog_open(GtkWidget *parent, const char *title,
                            const char *pattern, void (*func)(char *));
  void callback_gtk_file_dialog_open(GtkWidget *widget, gpointer gd);



/*--[ Confirmation Dialog ]---------------------------------------------------*/
  void gtk_confirmation_dialog_open(GtkWidget *parent, const char *title,
                                    const char *message, void (*func)(void));
  void callback_gtk_confirmation_dialog_open(GtkWidget *widget, gpointer gd);



/*--[ Message Dialog ]--------------------------------------------------------*/
  void gtk_message_dialog_open(GtkWidget *parent, const char *title, 
                               const char *message);



/*--[ fput_? Rouchines ]------------------------------------------------------*/
  void fput_l(const unsigned long number, FILE *fp);
  void fput_s(const unsigned short number, FILE *fp);
  void fput_str(const unsigned char *string, const int size, FILE *fp);



/*--[ Check Extention ]-------------------------------------------------------*/
  int chk_ext(const char *name, const char *ext);
  int strcmp_ci(const char *str1, const char *str2);



/*--[ Main ]------------------------------------------------------------------*/
int main(int argc, char *argv[])
{

  gtk_set_locale();
  gtk_init(&argc, &argv);

  int i, j, k;


/* initial settings */
  dotSize = 8;
  mainXSize = (dotSize + 1) * 5;
  mainYSize = (dotSize + 1) * 7;
  mainWidth = mainXSize * 16 + 1;
  mainHeight = mainYSize * 16 + 1;
  dotSize2 = dotSize * 8;
  subWidth = (dotSize2 + 1) * 5 + 1;
  subHeight = (dotSize2 + 1) * 7 + 1;


/* widget_new */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize(window);
    vb_menu = gtk_vbox_new(FALSE, 0);
      menu = gtk_menu_bar_new();
      hb_main = gtk_hbox_new(FALSE, 0);
        eb_main = gtk_event_box_new();
          da_main = gtk_drawing_area_new();
        vb_sub = gtk_vbox_new(FALSE, 0);
          lb_desc = gtk_label_new("");
          eb_sub = gtk_event_box_new();
            da_sub = gtk_drawing_area_new();


/* widget_setting */
  gtk_window_set_title(GTK_WINDOW(window), OCFE_TITLE);
  gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
  gtk_drawing_area_size(GTK_DRAWING_AREA(da_main), mainWidth, mainHeight);
  gtk_drawing_area_size(GTK_DRAWING_AREA(da_sub), subWidth, subHeight);
#if GTK_MAJOR_VERSION == 1
  gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menu), GTK_SHADOW_NONE);
#endif
  gtk_misc_set_alignment(GTK_MISC(lb_desc), 0, 0);

  /* menu_setting */
  GtkWidget *menu_file = gtk_menu_item_new_with_label("File(_F)");
    gtk_menu_bar_append(GTK_MENU_BAR(menu), menu_file);
    gtk_menu_item_parse_uline(menu_file);
    gtk_widget_show(menu_file);
  GtkWidget *menu_edit = gtk_menu_item_new_with_label("Edit(_E)");
    gtk_menu_bar_append(GTK_MENU_BAR(menu), menu_edit);
    gtk_menu_item_parse_uline(menu_edit);
    gtk_widget_show(menu_edit);
  GtkWidget *menu_help = gtk_menu_item_new_with_label("Help(_H)");
    gtk_menu_bar_append(GTK_MENU_BAR(menu), menu_help);
    gtk_menu_item_parse_uline(menu_help);
    gtk_widget_show(menu_help);

  /* File */
  GtkWidget *pop_file = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_file), pop_file);
    GtkWidget *file_open = gtk_menu_item_new_with_label("Open(_O)...");
      gtk_menu_append(GTK_MENU(pop_file), file_open);
      gtk_menu_item_parse_uline(file_open);
      gtk_widget_show(file_open);
    GtkWidget *file_save = gtk_menu_item_new_with_label("Save As(_S)...");
      gtk_menu_append(GTK_MENU(pop_file), file_save);
      gtk_menu_item_parse_uline(file_save);
      gtk_widget_show(file_save);
    GtkWidget *file_hr1  = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_file), file_hr1);
      gtk_widget_show(file_hr1);
    GtkWidget *file_comp = gtk_menu_item_new_with_label("Compile(_C)...");
      gtk_menu_append(GTK_MENU(pop_file), file_comp);
      gtk_menu_item_parse_uline(file_comp);
      gtk_widget_show(file_comp);
    GtkWidget *file_decp = gtk_menu_item_new_with_label("Decompile(_D)...");
      gtk_menu_append(GTK_MENU(pop_file), file_decp);
      gtk_menu_item_parse_uline(file_decp);
      gtk_widget_show(file_decp);
    GtkWidget *file_hr2  = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_file), file_hr2);
      gtk_widget_show(file_hr2);
    GtkWidget *file_img  = gtk_menu_item_new_with_label("Save As Image(_I)");
      gtk_menu_append(GTK_MENU(pop_file), file_img);
      gtk_menu_item_parse_uline(file_img);
      gtk_widget_show(file_img);
    GtkWidget *file_asm  = gtk_menu_item_new_with_label("Save As ASM(_A)");
      gtk_menu_append(GTK_MENU(pop_file), file_asm);
      gtk_menu_item_parse_uline(file_asm);
      gtk_widget_show(file_asm);
    GtkWidget *file_font  = gtk_menu_item_new_with_label("Save As Font(_F)");
      gtk_menu_append(GTK_MENU(pop_file), file_font);
      gtk_menu_item_parse_uline(file_font);
      gtk_widget_show(file_font);
    GtkWidget *file_hr3  = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_file), file_hr3);
      gtk_widget_show(file_hr3);
    GtkWidget *file_exit = gtk_menu_item_new_with_label("Exit(_X)");
      gtk_menu_append(GTK_MENU(pop_file), file_exit);
      gtk_menu_item_parse_uline(file_exit);
      gtk_widget_show(file_exit);

  /* File_IMG */
  GtkWidget *pop_file_img = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_img), pop_file_img);
    GtkWidget *file_img_XPM = gtk_menu_item_new_with_label("XPM(_X)...");
      gtk_menu_append(GTK_MENU(pop_file_img), file_img_XPM);
      gtk_menu_item_parse_uline(file_img_XPM);
      gtk_widget_show(file_img_XPM);
    GtkWidget *file_img_BMP = gtk_menu_item_new_with_label("BMP(_B)...");
      gtk_menu_append(GTK_MENU(pop_file_img), file_img_BMP);
      gtk_menu_item_parse_uline(file_img_BMP);
      gtk_widget_show(file_img_BMP);

  /* File_ASM */
  GtkWidget *pop_file_asm = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_asm), pop_file_asm);
    GtkWidget *file_asm_right = gtk_menu_item_new_with_label("Right Aligned(_R)...");
      gtk_menu_append(GTK_MENU(pop_file_asm), file_asm_right);
      gtk_menu_item_parse_uline(file_asm_right);
      gtk_widget_show(file_asm_right);
    GtkWidget *file_asm_left = gtk_menu_item_new_with_label("Left Aligned(_L)...");
      gtk_menu_append(GTK_MENU(pop_file_asm), file_asm_left);
      gtk_menu_item_parse_uline(file_asm_left);
      gtk_widget_show(file_asm_left);

  /* File_Font */
  GtkWidget *pop_file_font = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_font), pop_file_font);
    GtkWidget *file_font_BDF = gtk_menu_item_new_with_label("BDF[Beta](_B)...");
      gtk_menu_append(GTK_MENU(pop_file_font), file_font_BDF);
      gtk_menu_item_parse_uline(file_font_BDF);
      gtk_widget_show(file_font_BDF);

  /* Edit */
  GtkWidget *pop_edit = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_edit), pop_edit);
    GtkWidget *edit_cut = gtk_menu_item_new_with_label("Cut(_X)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_cut);
      gtk_menu_item_parse_uline(edit_cut);
      gtk_widget_show(edit_cut);
    GtkWidget *edit_copy = gtk_menu_item_new_with_label("Copy(_C)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_copy);
      gtk_menu_item_parse_uline(edit_copy);
      gtk_widget_show(edit_copy);
    GtkWidget *edit_paste = gtk_menu_item_new_with_label("Paste(_V)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_paste);
      gtk_menu_item_parse_uline(edit_paste);
      gtk_widget_show(edit_paste);
    GtkWidget *edit_hr1 = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_edit), edit_hr1);
      gtk_widget_show(edit_hr1);
    GtkWidget *edit_def = gtk_menu_item_new_with_label("Default(_D)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_def);
      gtk_menu_item_parse_uline(edit_def);
      gtk_widget_show(edit_def);
    GtkWidget *edit_clr = gtk_menu_item_new_with_label("Clear(_Q)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_clr);
      gtk_menu_item_parse_uline(edit_clr);
      gtk_widget_show(edit_clr);
    GtkWidget *edit_inv = gtk_menu_item_new_with_label("Inverse(_R)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_inv);
      gtk_menu_item_parse_uline(edit_inv);
      gtk_widget_show(edit_inv);
    GtkWidget *edit_hr2 = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_edit), edit_hr2);
      gtk_widget_show(edit_hr2);
    GtkWidget *edit_fh = gtk_menu_item_new_with_label("Flip Horizontal(_H)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_fh);
      gtk_menu_item_parse_uline(edit_fh);
      gtk_widget_show(edit_fh);
    GtkWidget *edit_fv = gtk_menu_item_new_with_label("Flip Vertical(_T)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_fv);
      gtk_menu_item_parse_uline(edit_fv);
      gtk_widget_show(edit_fv);
    GtkWidget *edit_rtt = gtk_menu_item_new_with_label("Rotate 180(_Y)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_rtt);
      gtk_menu_item_parse_uline(edit_rtt);
      gtk_widget_show(edit_rtt);
    GtkWidget *edit_hr3 = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_edit), edit_hr3);
      gtk_widget_show(edit_hr3);
    GtkWidget *edit_all = gtk_menu_item_new_with_label("ALL(_A)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_all);
      gtk_menu_item_parse_uline(edit_all);
      gtk_widget_show(edit_all);
    GtkWidget *edit_hr4 = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_edit), edit_hr4);
      gtk_widget_show(edit_hr4);
    GtkWidget *edit_size = gtk_menu_item_new_with_label("Dots Size(_S)");
      gtk_menu_append(GTK_MENU(pop_edit), edit_size);
      gtk_menu_item_parse_uline(edit_size);
      gtk_widget_show(edit_size);

  /* Edit_All */
  GtkWidget *pop_edit_all = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_all), pop_edit_all);
    GtkWidget *edit_all_def = gtk_menu_item_new_with_label("_Default...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_def);
      gtk_menu_item_parse_uline(edit_all_def);
      gtk_widget_show(edit_all_def);
    GtkWidget *edit_all_clr = gtk_menu_item_new_with_label("_Clear...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_clr);
      gtk_menu_item_parse_uline(edit_all_clr);
      gtk_widget_show(edit_all_clr);
    GtkWidget *edit_all_inv = gtk_menu_item_new_with_label("_Inverse...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_inv);
      gtk_menu_item_parse_uline(edit_all_inv);
      gtk_widget_show(edit_all_inv);
    GtkWidget *edit_all_hr1 = gtk_menu_item_new();
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_hr1);
      gtk_widget_show(edit_all_hr1);
    GtkWidget *edit_all_fh = gtk_menu_item_new_with_label("Flip _Horizontal...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_fh);
      gtk_menu_item_parse_uline(edit_all_fh);
      gtk_widget_show(edit_all_fh);
    GtkWidget *edit_all_fv = gtk_menu_item_new_with_label("Flip _Vertical...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_fv);
      gtk_menu_item_parse_uline(edit_all_fv);
      gtk_widget_show(edit_all_fv);
    GtkWidget *edit_all_rtt = gtk_menu_item_new_with_label("_Rotate 180...");
      gtk_menu_append(GTK_MENU(pop_edit_all), edit_all_rtt);
      gtk_menu_item_parse_uline(edit_all_rtt);
      gtk_widget_show(edit_all_rtt);

  /* Edit_Size */
  GSList *group_edit_size = NULL;
  GtkWidget *pop_edit_size = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_size), pop_edit_size);
    GtkWidget *edit_size_2 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_2px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_2));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_2);
      gtk_menu_item_parse_uline(edit_size_2);
      gtk_widget_show(edit_size_2);
    GtkWidget *edit_size_3 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_3px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_3));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_3);
      gtk_menu_item_parse_uline(edit_size_3);
      gtk_widget_show(edit_size_3);
    GtkWidget *edit_size_4 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_4px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_4));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_4);
      gtk_menu_item_parse_uline(edit_size_4);
      gtk_widget_show(edit_size_4);
    GtkWidget *edit_size_5 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_5px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_5));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_5);
      gtk_menu_item_parse_uline(edit_size_5);
      gtk_widget_show(edit_size_5);
    GtkWidget *edit_size_6 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_6px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_6));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_6);
      gtk_menu_item_parse_uline(edit_size_6);
      gtk_widget_show(edit_size_6);
    GtkWidget *edit_size_7 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_7px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_7));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_7);
      gtk_menu_item_parse_uline(edit_size_7);
      gtk_widget_show(edit_size_7);
    GtkWidget *edit_size_8 = gtk_radio_menu_item_new_with_label(
                               group_edit_size, "_8px");
      group_edit_size = gtk_radio_menu_item_group(
                               GTK_RADIO_MENU_ITEM(edit_size_8));
      gtk_menu_append(GTK_MENU(pop_edit_size), edit_size_8);
      gtk_menu_item_parse_uline(edit_size_8);
      gtk_widget_show(edit_size_8);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(edit_size_3), TRUE);

  /* Help */
  GtkWidget *pop_help = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_help), pop_help);
    GtkWidget *help_help = gtk_menu_item_new_with_label("Help(_H)...");
      gtk_menu_append(GTK_MENU(pop_help), help_help);
      gtk_menu_item_parse_uline(help_help);
      gtk_widget_show(help_help);
    GtkWidget *help_ver = gtk_menu_item_new_with_label("Version(_A)...");
      gtk_menu_append(GTK_MENU(pop_help), help_ver);
      gtk_menu_item_parse_uline(help_ver);
      gtk_widget_show(help_ver);


/* signal_connect */
  /* Window */
  gtk_signal_connect(GTK_OBJECT(window), "destroy", 
                                GTK_SIGNAL_FUNC(window_destroy), NULL);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
                                GTK_SIGNAL_FUNC(window_delete), NULL);
  gtk_signal_connect(GTK_OBJECT(window), "key_press_event", 
                                GTK_SIGNAL_FUNC(window_key_press), NULL);

  /* Drawing Area */
  gtk_signal_connect(GTK_OBJECT(da_main), "expose_event", 
                                GTK_SIGNAL_FUNC(da_main_expose), NULL);
  gtk_signal_connect(GTK_OBJECT(eb_main), "button_press_event", 
                                GTK_SIGNAL_FUNC(da_main_button_press), NULL);
  gtk_signal_connect(GTK_OBJECT(da_sub), "expose_event", 
                                GTK_SIGNAL_FUNC(da_sub_expose), NULL);
  gtk_signal_connect(GTK_OBJECT(eb_sub), "button_press_event", 
                                GTK_SIGNAL_FUNC(da_sub_button_press), NULL);

  /* File_open */
  gtk_signal_connect(GTK_OBJECT(file_open), "activate", 
                                GTK_SIGNAL_FUNC(file_open_clicked), NULL);
  /* File_save */
  gtk_signal_connect(GTK_OBJECT(file_save), "activate", 
                                GTK_SIGNAL_FUNC(file_save_clicked), NULL);
  /* File_comp */
  gtk_signal_connect(GTK_OBJECT(file_comp), "activate", 
                                GTK_SIGNAL_FUNC(file_comp_clicked), NULL);
  /* File_decp */
  gtk_signal_connect(GTK_OBJECT(file_decp), "activate", 
                                GTK_SIGNAL_FUNC(file_decp_clicked), NULL);
  /* File_img_XPM */
  gtk_signal_connect(GTK_OBJECT(file_img_XPM), "activate", 
                                GTK_SIGNAL_FUNC(file_img_XPM_clicked), NULL);
  /* File_img_BMP */
  gtk_signal_connect(GTK_OBJECT(file_img_BMP), "activate", 
                                GTK_SIGNAL_FUNC(file_img_BMP_clicked), NULL);
  /* File_asm_right */
  gtk_signal_connect(GTK_OBJECT(file_asm_right), "activate", 
                                GTK_SIGNAL_FUNC(file_asm_right_clicked), NULL);
  /* File_asm_left */
  gtk_signal_connect(GTK_OBJECT(file_asm_left), "activate", 
                                GTK_SIGNAL_FUNC(file_asm_left_clicked), NULL);
  /* File_font_BDF */
  gtk_signal_connect(GTK_OBJECT(file_font_BDF), "activate", 
                                GTK_SIGNAL_FUNC(file_font_BDF_clicked), NULL);
  /* File_exit */
  gtk_signal_connect(GTK_OBJECT(file_exit), "activate", 
                                GTK_SIGNAL_FUNC(window_destroy), NULL);

  /* Edit_cut */
  gtk_signal_connect(GTK_OBJECT(edit_cut), "activate",
                                GTK_SIGNAL_FUNC(edit_cut_clicked), NULL);
  /* Edit_copy */
  gtk_signal_connect(GTK_OBJECT(edit_copy), "activate",
                                GTK_SIGNAL_FUNC(edit_copy_clicked), NULL);
  /* Edit_paste */
  gtk_signal_connect(GTK_OBJECT(edit_paste), "activate",
                                GTK_SIGNAL_FUNC(edit_paste_clicked), NULL);
  /* Edit_def */
  gtk_signal_connect(GTK_OBJECT(edit_def), "activate",
                                GTK_SIGNAL_FUNC(edit_def_clicked), NULL);
  /* Edit_clr */
  gtk_signal_connect(GTK_OBJECT(edit_clr), "activate",
                                GTK_SIGNAL_FUNC(edit_clr_clicked), NULL);
  /* Edit_inv */
  gtk_signal_connect(GTK_OBJECT(edit_inv), "activate",
                                GTK_SIGNAL_FUNC(edit_inv_clicked), NULL);
  /* Edit_fh */
  gtk_signal_connect(GTK_OBJECT(edit_fh), "activate",
                                GTK_SIGNAL_FUNC(edit_fh_clicked), NULL);
  /* Edit_fv */
  gtk_signal_connect(GTK_OBJECT(edit_fv), "activate",
                                GTK_SIGNAL_FUNC(edit_fv_clicked), NULL);
  /* Edit_rtt */
  gtk_signal_connect(GTK_OBJECT(edit_rtt), "activate",
                                GTK_SIGNAL_FUNC(edit_rtt_clicked), NULL);
  /* Edit_All_def */
  gtk_signal_connect(GTK_OBJECT(edit_all_def), "activate",
                                GTK_SIGNAL_FUNC(edit_all_def_clicked), NULL);
  /* Edit_All_clr */
  gtk_signal_connect(GTK_OBJECT(edit_all_clr), "activate",
                                GTK_SIGNAL_FUNC(edit_all_clr_clicked), NULL);
  /* Edit_All_inv */
  gtk_signal_connect(GTK_OBJECT(edit_all_inv), "activate",
                                GTK_SIGNAL_FUNC(edit_all_inv_clicked), NULL);
  /* Edit_All_fh */
  gtk_signal_connect(GTK_OBJECT(edit_all_fh), "activate",
                                GTK_SIGNAL_FUNC(edit_all_fh_clicked), NULL);
  /* Edit_All_fv */
  gtk_signal_connect(GTK_OBJECT(edit_all_fv), "activate",
                                GTK_SIGNAL_FUNC(edit_all_fv_clicked), NULL);
  /* Edit_All_rtt */
  gtk_signal_connect(GTK_OBJECT(edit_all_rtt), "activate",
                                GTK_SIGNAL_FUNC(edit_all_rtt_clicked), NULL);
  /* Edit_Size_2 */
  gtk_signal_connect(GTK_OBJECT(edit_size_2), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked),
                                  GINT_TO_POINTER(2));
  /* Edit_Size_3 */
  gtk_signal_connect(GTK_OBJECT(edit_size_3), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(3)); 
  /* Edit_Size_4 */
  gtk_signal_connect(GTK_OBJECT(edit_size_4), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(4));
  /* Edit_Size_5 */
  gtk_signal_connect(GTK_OBJECT(edit_size_5), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(5));
  /* Edit_Size_6 */
  gtk_signal_connect(GTK_OBJECT(edit_size_6), "activate", 
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(6));
  /* Edit_Size_7 */
  gtk_signal_connect(GTK_OBJECT(edit_size_7), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(7));
  /* Edit_Size_8 */
  gtk_signal_connect(GTK_OBJECT(edit_size_8), "activate",
                                GTK_SIGNAL_FUNC(edit_size_clicked), 
                                  GINT_TO_POINTER(8));

  /* Help_help */
  gtk_signal_connect(GTK_OBJECT(help_help), "activate",
                                GTK_SIGNAL_FUNC(help_help_clicked), NULL);
  /* Help_ver */
  gtk_signal_connect(GTK_OBJECT(help_ver), "activate",
                                GTK_SIGNAL_FUNC(help_ver_clicked), NULL);


/* container_border_width */
  gtk_container_set_border_width(GTK_CONTAINER(hb_main), 10);
  gtk_container_set_border_width(GTK_CONTAINER(vb_sub), 10);


/* container_add */
  gtk_container_add(GTK_CONTAINER(window), vb_menu);
    gtk_container_add(GTK_CONTAINER(vb_menu), menu);
    gtk_container_add(GTK_CONTAINER(vb_menu), hb_main);
      gtk_container_add(GTK_CONTAINER(hb_main), eb_main);
        gtk_container_add(GTK_CONTAINER(eb_main), da_main);
      gtk_container_add(GTK_CONTAINER(hb_main), vb_sub);
        gtk_box_pack_start(GTK_BOX(vb_sub), lb_desc, FALSE, FALSE, 4);
        gtk_box_pack_start(GTK_BOX(vb_sub), eb_sub, FALSE, FALSE, 4);
          gtk_container_add(GTK_CONTAINER(eb_sub), da_sub);


/* gdk_setting */
  buf_main = gdk_pixmap_new(window->window, mainWidth, mainHeight, -1);
  buf_sub = gdk_pixmap_new(window->window, subWidth, subHeight, -1);

    for(i=0; i<6; i++)
    {
      gdk_color_parse(ColorNames[i], &Colors[i]);
      gdk_colormap_alloc_color(gtk_widget_get_colormap(window), &Colors[i],
                               FALSE, FALSE);
    }


/* val_setting */
  /* Clipboard */
    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          clipbrd[j][i] = 0;
        }
    }

  /* Load default font */
    for(i=0; i<256; i++)
    {
        for(j=0; j<7; j++)
        {
          unsigned char tmp = font_data[i*7+j];
            for(k=0; k<5; k++)
            {
              TIFonts[i].data[k][j] = ((tmp & 0x10)==0)? 0: 255;
              tmp <<= 1;
            }
        }
      TIFontd[i] = TIFonts[i];
    }


  /* Widget_style */
  ver_title_style = gtk_style_new();
  ver_title_style = gtk_style_copy(gtk_widget_get_style(lb_desc));
#if GTK_MAJOR_VERSION == 1
  ver_title_style->font = gdk_fontset_load(
                            "-*-*-medium-r-normal--24-*-*-*-*-*-iso8859-*");
#endif
#if GTK_MAJOR_VERSION == 2
  ver_title_style->font_desc = pango_font_description_from_string("Sans 18");
#endif
  ver_addr_style = gtk_style_new();
  ver_addr_style = gtk_style_copy(gtk_widget_get_style(lb_desc));
  ver_addr_style->fg[GTK_STATE_NORMAL] = Colors[COLOR_URI];


/* Other_setting */
  /* Set dots size */
  changeSize(3);

  /* Set description label size */
  int max_w = 0;
  GtkRequisition req;
    for(i=0; i<256; i++)
    {
      char dbuf[64];
      strcpy(dbuf, "000 ");
      strcat(dbuf, cDesc[i]);
      gtk_label_set_text(GTK_LABEL(lb_desc), dbuf);
      gtk_widget_size_request(lb_desc, &req);
        if(max_w < req.width)
        {
          max_w = req.width;
        }
    }
  gtk_widget_set_usize(lb_desc, max_w, req.height);
  gtk_label_set_text(GTK_LABEL(lb_desc), "0 (None)");

  /* Set icon */
  ico_pmap = gdk_pixmap_create_from_xpm_d(window->window, NULL, NULL, icon);
  gdk_window_set_icon(window->window, NULL, ico_pmap, ico_mask);
  gdk_window_set_icon_name(window->window, OCFE_APPNAME);


/* processing-arguments */
    if(argc >= 2)
    {
        if(access(argv[1], F_OK | R_OK) == 0)
        {
            if(chk_ext(argv[1], ".8xfont") == 0)
            {
              callback_file_open_clicked(argv[1]);
            }
            else if(chk_ext(argv[1], ".8xp") == 0)
            {
              callback_file_decp_clicked(argv[1]);
            }
        }
    }


/* widget_show */
    gtk_widget_show(vb_menu);
    gtk_widget_show(menu);
      gtk_widget_show(hb_main);
        gtk_widget_show(eb_main);
          gtk_widget_show(da_main);
        gtk_widget_show(vb_sub);
          gtk_widget_show(lb_desc);
          gtk_widget_show(eb_sub);
            gtk_widget_show(da_sub);
  gtk_widget_show(window);


  gtk_main();
  gtk_exit(0);

  return 0;

}



/*--[ Signal Functions ]------------------------------------------------------*/
gboolean window_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  return FALSE;
}

void window_destroy(GtkWidget *widget, gpointer gd)
{
  gtk_main_quit();
}

gboolean da_main_expose(GtkWidget *widget, GdkEventExpose *event, gpointer gd)
{
  redraw_main();
  return FALSE;
}

gboolean da_main_button_press(GtkWidget *widget, 
                              GdkEventButton *event, gpointer gd)
{
  GdkEventButton *button = (GdkEventButton *)event;
    if(button->type != GDK_BUTTON_PRESS)
    {
      return FALSE;
    }

  int x = (int)(button->x/mainXSize);
  int y = (int)(button->y/mainYSize);
    if(x >= 15)
    {
      x = 15;
    }
    if(y >= 15)
    {
      y = 15;
    }
  activeChar = y * 16 + x;

    if(button->button == 1)
    {
      int dx = (button->x - (activeChar % 16) * mainXSize) / (dotSize+1);
      int dy = (button->y - (activeChar / 16) * mainYSize) / (dotSize+1);
        if(dx >= 4)
        {
          dx = 4;
        }
        if(dy >= 6)
        {
          dy = 6;
        }
      TIFonts[activeChar].data[dx][dy] ^= 255;
    }

  redraw_all();
  return FALSE;
}

gboolean da_sub_expose(GtkWidget *widget, GdkEventExpose *event, gpointer gd)
{
  redraw_sub();
  return FALSE;
}

gboolean da_sub_button_press(GtkWidget *widget, 
                             GdkEventButton *event, gpointer gd)
{
  GdkEventButton *button = (GdkEventButton *)event;
    if(button->type != GDK_BUTTON_PRESS)
    {
      return FALSE;
    }

  int x = button->x/(dotSize2+1);
  int y = button->y/(dotSize2+1);
    if(x >= 4) x = 4;
    if(y >= 6) y = 6;

  TIFonts[activeChar].data[x][y] ^= 255;
  redraw_all();
  return FALSE;
}

gboolean window_key_press(GtkWidget *widget, GdkEventKey *event, gpointer gd)
{
  key_press(event->keyval, event->state);
  return FALSE;
}

void file_open_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Open from 8xfont file...", 
                       "*.8xfont", callback_file_open_clicked);
}

void file_save_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as 8xfont file...", 
                       "*.8xfont", callback_file_save_clicked);
}

void file_comp_clicked(GtkMenuItem *item, gpointer gd)
{
  GtkRequisition req;
  GtkWidget *dialog = gtk_dialog_new();
  GtkWidget *btn_ok = gtk_button_new_with_label("OK");
  GtkWidget *btn_ng = gtk_button_new_with_label("Cancel");
  GtkWidget *lb_exp = gtk_label_new("Input font name:");
  GtkWidget *tt_name = gtk_entry_new();

  gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);

  gtk_entry_set_text(GTK_ENTRY(tt_name), "");
  gtk_misc_set_alignment(GTK_MISC(lb_exp), 0, 0);
  gtk_window_set_title(GTK_WINDOW(dialog), "Font Name");
  gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

  gtk_signal_connect(GTK_OBJECT(btn_ok), "clicked",
                     GTK_SIGNAL_FUNC(file_comp_clicked_2nd), (gpointer)tt_name);
  gtk_signal_connect_object(GTK_OBJECT(btn_ok), "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
  gtk_signal_connect_object(GTK_OBJECT(btn_ng), "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), lb_exp);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), tt_name);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), btn_ok,
                     FALSE, FALSE, 0);
  gtk_widget_size_request(btn_ok, &req);
  req.width += 64;
  gtk_widget_set_usize(btn_ok, req.width, req.height);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), btn_ng,
                     FALSE, FALSE, 0);
  gtk_widget_size_request(btn_ng, &req);
  req.width += 48;
  gtk_widget_set_usize(btn_ng, req.width, req.height);

  gtk_widget_show(btn_ok);
  gtk_widget_show(btn_ng);
  gtk_widget_show(lb_exp);
  gtk_widget_show(tt_name);
  gtk_widget_show_now(dialog);
  gdk_window_raise(dialog->window);
  gtk_widget_grab_focus(tt_name);
}
void file_comp_clicked_2nd(GtkWidget *widget, gpointer gd)
{

  GtkWidget *entry = GTK_WIDGET(gd);
  char *name = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

  int i;
  int len = strlen(name);

  memset(compname, '\0', 8);
    if(len == 0)
    {
      g_free(name);
      gtk_message_dialog_open(window, "Compile Error",
                              "Please enter font name.");
      return ;
    }
    if(len<0 || 8<len)
    {
      g_free(name);
      gtk_message_dialog_open(window, "Compile Error", 
                              "Please enter 1 to 8 character(s).");
      return ;
    }
    for(i=0; i<len; i++)
    {
        if(isalnum(name[i]) == 0)
        {
          g_free(name);
          gtk_message_dialog_open(window, "Compile Error",
                                  "Including invalid character(s).\n"
                                    "Please use [A-Za-z0-9].");
          return ;
        }
      compname[i] = toupper(name[i]);
    }
    if(isalpha(compname[0]) == 0)
    {
      g_free(name);
      gtk_message_dialog_open(window, "Compile Error", 
                              "The first character must be an alphabet.");
      return ;
    }

  char buf[16];
  strcpy(buf, name);
  g_free(name);
  strcat(buf, ".8xp");
  gtk_file_dialog_open(window, "Save as compiled font...",
                       buf, callback_file_comp_clicked);
}

void file_decp_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Open from compiled font...", 
                       "*.8xp", callback_file_decp_clicked);
}

void file_asm_right_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as ASM (Right Aligned)...",
                       "*.z80", callback_file_asm_right_clicked);
}

void file_asm_left_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as ASM (Left Aligned)...", 
                       "*.z80", callback_file_asm_left_clicked);
}

void file_img_XPM_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as XPM...", 
                       "*.xpm", callback_file_img_XPM_clicked);
}

void file_img_BMP_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as BMP...", 
                       "*.bmp", callback_file_img_BMP_clicked);
}

void file_font_BDF_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_file_dialog_open(window, "Save as BDF...", 
                       "*.bdf", callback_file_font_BDF_clicked);
}

void edit_cut_clicked(GtkMenuItem *item, gpointer gd)
{
  char_cut();
  redraw_all();
}

void edit_copy_clicked(GtkMenuItem *item, gpointer gd)
{
  char_copy();
  redraw_all();
}

void edit_paste_clicked(GtkMenuItem *item, gpointer gd)
{
  char_paste();
  redraw_all();
}

void edit_def_clicked(GtkMenuItem *item, gpointer gd)
{
  char_def();
  redraw_all();
}

void edit_clr_clicked(GtkMenuItem *item, gpointer gd)
{
  char_clr();
  redraw_all();
}

void edit_inv_clicked(GtkMenuItem *item, gpointer gd)
{
  char_inv();
  redraw_all();
}

void edit_fh_clicked(GtkMenuItem *item, gpointer gd)
{
  char_fh();
  redraw_all();
}

void edit_fv_clicked(GtkMenuItem *item, gpointer gd)
{
  char_fv();
  redraw_all();
}

void edit_rtt_clicked(GtkMenuItem *item, gpointer gd)
{
  char_rtt();
  redraw_all();
}

void edit_all_def_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE, 
                               CONFIRMATION_MSG, char_all_def);
}

void edit_all_clr_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE,
                               CONFIRMATION_MSG, char_all_clr);
}

void edit_all_inv_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE, 
                               CONFIRMATION_MSG, char_all_inv);
}

void edit_all_fh_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE,
                               CONFIRMATION_MSG, char_all_fh);
}

void edit_all_fv_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE, 
                               CONFIRMATION_MSG, char_all_fv);
}

void edit_all_rtt_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_confirmation_dialog_open(window, CONFIRMATION_TITLE, 
                               CONFIRMATION_MSG, char_all_rtt);
}

void edit_size_clicked(GtkMenuItem *item, gpointer gd)
{
  changeSize(GPOINTER_TO_INT(gd));
}

void help_help_clicked(GtkMenuItem *item, gpointer gd)
{
  gtk_message_dialog_open(window, "Help", 
    "Mouse:\n"
    "    Left click - Edit a pixel\n"
    "    Right click:\n"
    "        In the left area - Set a cursor\n"
    "        In the right area - Edit a pixel\n\n"
    "Keys:\n"
    "    Allow keys - Move the cursor\n"
    "    A,S,D,W key - Move the cursor\n"
    "    Ctrl+X - Cut\n"
    "    Ctrl+C - Copy\n"
    "    Ctrl+V - Paste\n"
    "    Ctrl+D - Default\n"
    "    Ctrl+R - Inverse\n"
    "    Ctrl+Q - Clear\n"
    "    Ctrl+H - Flip Horizontal\n"
    "    Ctrl+T - Flip Vertical\n"
    "    Ctrl+Y - Rotate 180\n"
  );
}

void help_ver_clicked(GtkMenuItem *item, gpointer gd)
{
  GtkRequisition req;
  GtkWidget *dialog = gtk_dialog_new();
  GtkWidget *hb_dlg = gtk_hbox_new(FALSE, 4);
  GtkWidget *vb_icon = gtk_vbox_new(FALSE, 0);
  GdkBitmap *mask;
  GdkPixmap *pmap = gdk_pixmap_create_from_xpm_d(window->window, &mask, NULL, icon);
  GtkWidget *pm_icon = gtk_pixmap_new(pmap, mask);
  GtkWidget *vb_dlg = gtk_vbox_new(FALSE, 4);
  GtkWidget *lb_title = gtk_label_new(OCFE_TITLE);
  GtkWidget *lb_help = gtk_label_new(OCFE_HELP);
  GtkWidget *lb_addr = gtk_label_new(OCFE_AUTHOR_ADDRESS);
  GtkWidget *lb_cr = gtk_label_new(OCFE_AUTHOR_NAME);
  GtkWidget *btn_ok = gtk_button_new_with_label("OK");

  gtk_widget_set_style(lb_title, ver_title_style);
  gtk_label_set_uline(lb_addr);
  gtk_widget_set_style(lb_addr, ver_addr_style);

  gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
  gtk_container_set_border_width(GTK_CONTAINER(vb_dlg), 5);
  gtk_container_set_border_width(GTK_CONTAINER(vb_icon), 5);
  gtk_container_set_border_width(GTK_CONTAINER(hb_dlg), 10);
  gtk_misc_set_alignment(GTK_MISC(lb_title), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(lb_help), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(lb_addr), 0, 0);
  gtk_misc_set_padding(GTK_MISC(lb_addr), 0, 6);
  gtk_label_set_justify(GTK_LABEL(lb_addr), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(lb_cr), 1, 0);
  gtk_window_set_title(GTK_WINDOW(dialog), "Version Information");
  gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

  gtk_signal_connect_object(GTK_OBJECT(btn_ok), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                              (gpointer)dialog);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hb_dlg);
  gtk_container_add(GTK_CONTAINER(hb_dlg), vb_icon);
  gtk_box_pack_start(GTK_BOX(vb_icon), pm_icon, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(hb_dlg), vb_dlg);
  gtk_container_add(GTK_CONTAINER(vb_dlg), lb_title);
  gtk_container_add(GTK_CONTAINER(vb_dlg), lb_help);
  gtk_container_add(GTK_CONTAINER(vb_dlg), lb_addr);
  gtk_container_add(GTK_CONTAINER(vb_dlg), lb_cr);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), btn_ok,
                     FALSE, FALSE, 0);
  gtk_widget_size_request(btn_ok, &req);
  req.width += 64;
  gtk_widget_set_usize(btn_ok, req.width, req.height);

  gtk_widget_show(hb_dlg);
  gtk_widget_show(vb_icon);
  gtk_widget_show(pm_icon);
  gtk_widget_show(vb_dlg);
  gtk_widget_show(lb_title);
  gtk_widget_show(lb_help);
  gtk_widget_show(lb_addr);
  gtk_widget_show(lb_cr);
  gtk_widget_show(btn_ok);
  gtk_widget_show_now(dialog);
  gdk_window_raise(dialog->window);
  gtk_widget_grab_focus(btn_ok);
}



/*--[ Callback Functions ]----------------------------------------------------*/
void redraw_main(void)
{

  GdkGC *gc_main = gdk_gc_new(buf_main);
  int i, j, k;

  gdk_gc_set_foreground(gc_main, &Colors[COLOR_BG]);
  gdk_draw_rectangle(buf_main, gc_main, TRUE, 0, 0, mainWidth, mainHeight);

  gdk_gc_set_foreground(gc_main, &Colors[COLOR_BORDER]);
    for(i=0; i<=mainWidth; i+=(dotSize+1))
    {
      gdk_draw_line(buf_main, gc_main, i, 0, i, mainHeight);
    }
    for(i=0; i<=mainHeight; i+=(dotSize+1))
    {
      gdk_draw_line(buf_main, gc_main, 0, i, mainWidth, i);
    }

  gdk_gc_set_foreground(gc_main, &Colors[COLOR_FRAME]);
    for(i=0; i<=mainWidth; i+=mainXSize)
    {
      gdk_draw_line(buf_main, gc_main, i, 0, i, mainHeight);
    }
    for(i=0; i<=mainHeight; i+=mainYSize)
    {
      gdk_draw_line(buf_main, gc_main, 0, i, mainWidth, i);
    }

  gdk_gc_set_foreground(gc_main, &Colors[COLOR_FG]);
    for(i=0; i<256; i++)
    {
      int activeX = (i % 16) * mainXSize;
      int activeY = (i / 16) * mainYSize;
        for(j=0; j<7; j++)
        {
           for(k=0; k<5; k++)
           {
               if(TIFonts[i].data[k][j] != 0)
               {
                 gdk_draw_rectangle(buf_main, gc_main, TRUE, 
                                    activeX+(dotSize+1)*k+1, 
                                    activeY+(dotSize+1)*j+1,
                                    dotSize, dotSize);
               }
           }
        }
    }

  gdk_gc_set_foreground(gc_main, &Colors[COLOR_ACTIVE]);
  int frameX = (activeChar % 16) * mainXSize;
  int frameY = (activeChar / 16) * mainYSize;
  gdk_draw_rectangle(buf_main, gc_main, FALSE, 
                     frameX, frameY, mainXSize, mainYSize);
  gdk_draw_pixmap(da_main->window, gc_main, buf_main, 
                  0, 0, 0, 0, mainWidth, mainHeight);
}

void redraw_sub(void)
{
  GdkGC *gc_sub = gdk_gc_new(buf_sub);
  int i, j;

  gdk_gc_set_foreground(gc_sub, &Colors[COLOR_BG]);
  gdk_draw_rectangle(buf_sub, gc_sub, TRUE, 0, 0, subWidth, subHeight);

  gdk_gc_set_foreground(gc_sub, &Colors[COLOR_BORDER]);
    for(i=0; i<=subWidth; i+=(dotSize2+1))
    {
      gdk_draw_line(buf_sub, gc_sub, i, 0, i, subHeight);
    }
    for(i=0; i<=subHeight; i+=(dotSize2+1))
    {
        if(i==(dotSize2+1)*2)
        {
          gdk_gc_set_foreground(gc_sub, &Colors[COLOR_FG]);
          gdk_draw_line(buf_sub, gc_sub, 0, i, subWidth, i);
          gdk_gc_set_foreground(gc_sub, &Colors[COLOR_BORDER]);
        }
        else
        {
          gdk_draw_line(buf_sub, gc_sub, 0, i, subWidth, i);
        }
    }

  gdk_gc_set_foreground(gc_sub, &Colors[COLOR_FG]);
    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
            if(TIFonts[activeChar].data[j][i] != 0)
            {
              gdk_draw_rectangle(buf_sub, gc_sub, TRUE, 
                (dotSize2+1)*j+1, (dotSize2+1)*i+1, dotSize2, dotSize2);
            }
        }
    }

  gdk_gc_set_foreground(gc_sub, &Colors[COLOR_FG]);
  gdk_draw_rectangle(buf_sub, gc_sub, FALSE, 0, 0, subWidth-1, subHeight-1);

  gdk_draw_pixmap(da_sub->window, gc_sub, buf_sub,
                  0, 0, 0, 0, subWidth, subHeight);
}

void redraw_desc(void)
{
  char buf[128];
  sprintf(buf, "%d ", activeChar);
  strcat(buf, cDesc[activeChar]);
  gtk_label_set_text(GTK_LABEL(lb_desc), buf);
}

void redraw_all(void)
{
  redraw_main();
  redraw_sub();
  redraw_desc();
}

void key_press(int key, int state)
{
    if((state & GDK_CONTROL_MASK) != 0)
    {
        switch(key)
        {
          case GDK_x:
            char_cut();
            break;

          case GDK_c:
            char_copy();
            break;

          case GDK_v:
            char_paste();
            break;

          case GDK_d:
            char_def();
            break;

          case GDK_q:
            char_clr();
            break;

          case GDK_r:
            char_inv();
            break;

          case GDK_h:
            char_fh();
            break;

          case GDK_t:
            char_fv();
            break;

          case GDK_y:
            char_rtt();
            break;

          default:
            break;
        }
    }
    else
    {
        switch(key)
        {
          case GDK_Up:
          case GDK_w:
              if(activeChar >= 16)
              {
                activeChar -= 16;
              }
            break;

          case GDK_Down:
          case GDK_s:
              if(activeChar <= 239)
              {
                activeChar += 16;
              }
            break;
    
          case GDK_Left:
          case GDK_a:
              if(activeChar % 16 != 0)
              {
                activeChar--;
              } 
            break;
    
          case GDK_Right:
          case GDK_d:
              if(activeChar % 16 != 15)
              {
                activeChar++;
              } 
            break;
    
          default:
            break;
        }
    }

  redraw_all();
}

void changeSize(int size)
{
  dotSize = size;
  mainXSize = (dotSize + 1) * 5;
  mainYSize = (dotSize + 1) * 7;
  mainWidth = mainXSize * 16 + 1;
  mainHeight = mainYSize * 16 + 1;
  dotSize2 = dotSize * 8;
  subWidth = (dotSize2 + 1) * 5 + 1;
  subHeight = (dotSize2 + 1) * 7 + 1;

  gtk_widget_set_usize(da_main, mainWidth, mainHeight);
  gtk_drawing_area_size(GTK_DRAWING_AREA(da_main), mainWidth, mainHeight);
  gtk_widget_set_usize(da_sub, subWidth, subHeight);
  gtk_drawing_area_size(GTK_DRAWING_AREA(da_sub), subWidth, subHeight);

  redraw_all();
}

void callback_file_open_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "rt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k;
  char buf[512];
    for(i=0; i<256; i++)
    {
        for(j=0; j<7; j++)
        {
            for(k=0; k<5; k++)
            {
              fgets(buf, 512-1, fp);
              TIFonts[i].data[k][j] = (buf[1] == '0')? 255: 0;
            }
        }
    }
  fclose(fp);

  redraw_all();
}

void callback_file_save_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k;
    for(i=0; i<256; i++)
    {
        for(j=0; j<7; j++)
        {
            for(k=0; k<5; k++)
            {
              fprintf(fp, " %d \n", (TIFonts[i].data[k][j] == 0)? 255: 0);
            }
        }
    }

  fclose(fp);
}

void callback_file_comp_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wb")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k;
  unsigned char header[11] = {0x2A, 0x2A, 0x54, 0x49, 0x38,
                              0x33, 0x46, 0x2A, 0x1A, 0x0A, 0x00};
  unsigned char comment[41];
  char *comment_ocfe = "Generated by " OCFE_APPNAME;
  memset(comment, 0x00, 41); strcpy(comment, comment_ocfe);
  unsigned char delimiter = 0x00;
  short datasize1 = 11+256*7 + 19;
  short random1 = 0x00*256 + 0x0D;
  short datasize2 = 11+256*7 + 2;
  unsigned char protectedd = 6;
  unsigned char name[8];
  memcpy(name, compname, 8);
  short random2 = 0x00*256+0x00;
  short datasize3 = 11+256*7 + 2;
  short datasize4 = 11+256*7 + 0;
  unsigned char fontheader[11] = {0xBB, 0x6D, 0xC9, 0x6F, 0x6D,
                                  0x6E, 0x69, 0x63, 0x61, 0x6C, 0x63};
  short checksum;
  int chk = 0;
  chk += random1 % 256;
  chk += random1 / 256;
  chk += datasize2 % 256;
  chk += datasize2 / 256;
  chk += protectedd;
  chk += random2 % 256;
  chk += random2 / 256;
  chk += datasize3 % 256;
  chk += datasize3 / 256;
  chk += datasize4 % 256;
  chk += datasize4 / 256;
    for(i=0; i<8; i++)
    {
      chk += name[i];
    }
    for(i=0; i<11; i++)
    {
      chk += fontheader[i];
    }
  fput_str(header, 11, fp);
  fput_str(comment, 41, fp);
  fputc(delimiter, fp);
  fput_s(datasize1, fp);
  fput_s(random1, fp);
  fput_s(datasize2, fp);
  fputc(protectedd, fp);
  fput_str(name, 8, fp);
  fput_s(random2, fp);
  fput_s(datasize3, fp);
  fput_s(datasize4, fp);
  fput_str(fontheader, 11, fp);
    for(i=0; i<256; i++)
    {
        for(j=0; j<7; j++)
        {
          unsigned char tmp = 0;
          int p = 16;
            for(k=0; k<5; k++)
            {
              tmp += ((TIFonts[i].data[k][j] == 0)? 0: 1) * p;
              p /= 2;
            }
          fputc(tmp, fp);
          chk += tmp;
        }
    }
  checksum = (short)chk;
  fput_s(checksum, fp);

  fclose(fp);
}

void callback_file_decp_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "rb")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k, tmp;
    for(i=0; i<85; i++)
    {
      tmp = fgetc(fp);
    }
    for(i=0; i<256; i++)
    {
        for(j=0; j<7; j++)
        {
          tmp = fgetc(fp);
            for(k=0; k<5; k++)
            {
              TIFonts[i].data[4-k][j] = (tmp % 2 == 0)? 0: 255;
              tmp /= 2;
            }
        }
    }
  fclose(fp);

  redraw_all();
}

void callback_file_img_XPM_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k, n;
  fprintf(fp, "/* XPM */\n");
  fprintf(fp, "/* This file was generated by " OCFE_APPNAME "*/\n");
  fprintf(fp, "static char *omnicalc_font_xpm[] = {\n");
  fprintf(fp, "\"97 129 2 1\",\n");
  fprintf(fp, "\"       c white\",\n");
  fprintf(fp, "\"@      c black\",\n");
  fprintf(fp, "\"");
    for(i=0; i<97; i++)
    {
      fputc(' ', fp);
    }
  fprintf(fp, "\",\n");
    for(i=0; i<16; i++)
    {
        for(j=0; j<8; j++)
        {
          fprintf(fp, "\" ");
            for(k=0; k<16; k++)
            {
                for(n=0; n<5; n++)
                {
                    if(j != 7)
                    {
                      fputc((TIFonts[i*16+k].data[n][j] == 0)? ' ': '@', fp);
                    }
                    else
                    {
                      fputc(' ', fp);
                    }
                }
              fputc(' ', fp);
            }
          fprintf(fp, "\"%s\n", (i==15 && j==7)? "": ",");
        }
    }
  fprintf(fp, "};\n");
  fclose(fp);
}
void callback_file_img_BMP_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wb")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k, n;
  fprintf(fp, "BM");                  /* Type          */
  fput_l(14 + 40 + (97*3+1)*129, fp); /* FileSize      */
  fput_s(0x0000, fp);                 /* Reserved1     */
  fput_s(0x0000, fp);                 /* Reserved2     */
  fput_l(14 + 40, fp);                /* Offset        */
  fput_l(40, fp);                     /* StructureSize */
  fput_l(97, fp);                     /* Width         */
  fput_l(129, fp);                    /* Height        */
  fput_s(1, fp);                      /* Planes        */
  fput_s(24, fp);                     /* BitCount      */
  fput_l(0, fp);                      /* Compression   */
  fput_l((97*3+1)*129, fp);           /* SizeImage     */
  fput_l(0, fp);                      /* XPimPerMeter  */
  fput_l(0, fp);                      /* YPimPerMeter  */
  fput_l(0, fp);                      /* ClrUsed       */
  fput_l(0, fp);                      /* ClrImportant  */
    for(i=0; i<16; i++)
    {
        for(j=0; j<8; j++)
        {
          fputc(0xFF, fp);
          fputc(0xFF, fp);
          fputc(0xFF, fp);
            for(k=0; k<16; k++)
            {
                for(n=0; n<5; n++)
                {
                    if(j != 0)
                    {
                        if(TIFonts[(15-i)*16+k].data[n][(7-j)] == 0)
                        {
                          fputc(0xFF, fp);
                          fputc(0xFF, fp);
                          fputc(0xFF, fp);
                        }
                        else
                        {
                          fputc(0x00, fp);
                          fputc(0x00, fp);
                          fputc(0x00, fp);
                        }
                    }
                    else
                    {
                      fputc(0xFF, fp);
                      fputc(0xFF, fp);
                      fputc(0xFF, fp);
                    }
                }
              fputc(0xFF, fp);
              fputc(0xFF, fp);
              fputc(0xFF, fp);
            }
          fputc(0x00, fp);
        }
    }
    for(i=0; i<97; i++)
    {
      fputc(0xFF, fp);
      fputc(0xFF, fp);
      fputc(0xFF, fp);
    }
  fputc(0x00, fp);
  fclose(fp);
}

void callback_file_font_BDF_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
      return ;
    }

  int i, j, k;
  const char *hex = "0123456789ABCDEF";
  fprintf(fp, "STARTFONT 2.1\n");
  fprintf(fp, "COMMENT This file was generated by " OCFE_APPNAME "\n");
  fprintf(fp, "FONT -Omnicalc-Fixed-Medium-R-Normal--16-160-75-75-C-120-ISO8859-1\n");
  fprintf(fp, "SIZE 16 75 75\n");
  fprintf(fp, "FONTBOUNDINGBOX 12 16 0 -2\n");
  fprintf(fp, "STARTPROPERTIES 19\n");
  fprintf(fp, "FONT_ASCENT 14\n");
  fprintf(fp, "FONT_DESCENT 2\n");
  fprintf(fp, "DEFAULT_CHAR 32\n");
  fprintf(fp, "COPYRIGHT \"Public Domain\"\n");
  fprintf(fp, "FONTNAME_REGISTRY \"\"\n");
  fprintf(fp, "FOUNDRY \"Omnicalc\"\n");
  fprintf(fp, "FAMILY_NAME \"Fixed\"\n");
  fprintf(fp, "WEIGHT_NAME \"Medium\"\n");
  fprintf(fp, "SLANT \"R\"\n");
  fprintf(fp, "SETWIDTH_NAME \"Normal\"\n");
  fprintf(fp, "ADD_STYLE_NAME \"\"\n");
  fprintf(fp, "PIXEL_SIZE 16\n");
  fprintf(fp, "POINT_SIZE 160\n");
  fprintf(fp, "RESOLUTION_X 75\n");
  fprintf(fp, "RESOLUTION_Y 75\n");
  fprintf(fp, "SPACING \"C\"\n");
  fprintf(fp, "AVERAGE_WIDTH 120\n");
  fprintf(fp, "CHARSET_REGISTRY \"ISO8859\"\n");
  fprintf(fp, "CHARSET_ENCODING \"1\"\n");
  fprintf(fp, "ENDPROPERTIES\n");
  fprintf(fp, "CHARS 256\n");
    for(i=0; i<256; i++)
    {
      fprintf(fp, "STARTCHAR char%d\n", i);
      fprintf(fp, "ENCODING %d\n", i);
      fprintf(fp, "SWIDTH 600 0\n");
      fprintf(fp, "DWIDTH 16 0\n");
      fprintf(fp, "BBX 12 16 0 0\n");
      fprintf(fp, "BITMAP\n");
      fprintf(fp, "0000\n");
      fprintf(fp, "0000\n");
        for(j=0; j<7; j++)
        {
          int bdf[4];
          bdf[0]  = ((TIFonts[i].data[0][j] == 0)? 0: 1) * 12; /* 0b1100 */
          bdf[0] += ((TIFonts[i].data[1][j] == 0)? 0: 1) * 3;  /* 0b0011 */
          bdf[1]  = ((TIFonts[i].data[2][j] == 0)? 0: 1) * 12; /* 0b1100 */
          bdf[1] += ((TIFonts[i].data[3][j] == 0)? 0: 1) * 3;  /* 0b0011 */
          bdf[2]  = ((TIFonts[i].data[4][j] == 0)? 0: 1) * 12; /* 0b1100 */
          bdf[3]  = 0;
            for(k=0; k<4; k++)
            {
              fputc(hex[bdf[k]], fp);
            }
          fputc('\n', fp);
            for(k=0; k<4; k++)
            {
              fputc(hex[bdf[k]], fp);
            }
          fputc('\n', fp);
        }
      fprintf(fp, "ENDCHAR\n");
    }
  fprintf(fp, "ENDFONT\n");
  fclose(fp);
}

void callback_file_asm_right_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
    }

  int i, j, k;
  fprintf(fp, ";This file was generated by " OCFE_APPNAME "\n\n");
    for(i=0; i<256; i++)
    {
      fprintf(fp, "chr%03d:", i);
        for(j=0; j<7; j++)
        {
          fprintf(fp, "\t.db %%000");
            for(k=0; k<5; k++)
            {
              fprintf(fp, "%d", (TIFonts[i].data[k][j] == 0)? 0: 1);
            }
          fprintf(fp, "\n");
        }
      fprintf(fp, "\n");
    }
  fclose(fp);
}

void callback_file_asm_left_clicked(char *filename)
{
  FILE *fp;
    if((fp = fopen(filename, "wt")) == NULL)
    {
      g_print(ERROR_COULDNOT_OPEN);
    }

  int i, j, k;
  fprintf(fp, ";This file was generated by " OCFE_APPNAME "\n\n");
    for(i=0; i<256; i++)
    {
      fprintf(fp, "chr%03d:", i);
        for(j=0; j<7; j++)
        {
          fprintf(fp, "\t.db %%");
            for(k=0; k<5; k++)
            {
              fprintf(fp, "%d", (TIFonts[i].data[k][j] == 0)? 0: 1);
            }
          fprintf(fp, "000\n");
        }
      fprintf(fp, "\n");
    }
  fclose(fp);
}

void char_cut(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          clipbrd[j][i] = TIFonts[activeChar].data[j][i];
          TIFonts[activeChar].data[j][i] = 0;
        }
    }
}

void char_copy(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          clipbrd[j][i] = TIFonts[activeChar].data[j][i];
        }
    }
}

void char_paste(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          TIFonts[activeChar].data[j][i] = clipbrd[j][i];
        }
    }
}

void char_def(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          TIFonts[activeChar].data[j][i] = TIFontd[activeChar].data[j][i];
        }
    }
}

void char_clr(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          TIFonts[activeChar].data[j][i] = 0;
        }
    }
}

void char_inv(void)
{
  int i, j;

    for(i=0; i<7; i++)
    {
        for(j=0; j<5; j++)
        {
          TIFonts[activeChar].data[j][i] ^= 255;
        }
    }
}

void char_fh(void)
{
  int i, j;
  unsigned char t;

    for(i=0; i<7; i++)
    {
        for(j=0; j<2; j++)
        {
          t = TIFonts[activeChar].data[j][i];
          TIFonts[activeChar].data[j][i] = TIFonts[activeChar].data[4-j][i];
          TIFonts[activeChar].data[4-j][i] = t;
        }
    }
}

void char_fv(void)
{
  int i, j;
  unsigned char t;

    for(j=0; j<5; j++)
    {
        for(i=0; i<3; i++)
        {
          t = TIFonts[activeChar].data[j][i];
          TIFonts[activeChar].data[j][i] = TIFonts[activeChar].data[j][6-i];
          TIFonts[activeChar].data[j][6-i] = t;
        }
    }
}

void char_rtt(void)
{
  char_fh();
  char_fv();
}

void char_all_def(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_def();
    }
  activeChar = tmpChar;

  redraw_all();
}

void char_all_clr(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_clr();
    }
  activeChar = tmpChar;

  redraw_all();
}

void char_all_inv(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_inv();
    }
  activeChar = tmpChar;

  redraw_all();
}

void char_all_fh(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_fh();
    }
  activeChar = tmpChar;

  redraw_all();
}

void char_all_fv(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_fv();
    }
  activeChar = tmpChar;

  redraw_all();
}

void char_all_rtt(void)
{
  int tmpChar = activeChar;

    for(activeChar=0; activeChar<256; activeChar++)
    {
      char_rtt();
    }
  activeChar = tmpChar;

  redraw_all();
}



/*--[ Addtional Functions ]---------------------------------------------------*/
void gtk_menu_item_parse_uline(GtkWidget *item)
{
  GtkWidget *label = GTK_BIN(item)->child;
  char *text;
  gtk_label_get(GTK_LABEL(label), &text);
  gtk_label_parse_uline(GTK_LABEL(label), text);
}

void gtk_label_set_uline(GtkWidget *label)
{
  char *text;
  gtk_label_get(GTK_LABEL(label), &text);
  char uline[512];
  memset(uline, '\0', 512);
  memset(uline, '_', strlen(text));
  gtk_label_set_pattern(GTK_LABEL(label), uline);
}



/*--[ File Dialog ]-----------------------------------------------------------*/
void gtk_file_dialog_open(GtkWidget *parent, const char *title, 
                          const char *pattern, void (*func)(char *))
{
  filesel = gtk_file_selection_new(title);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
                     "clicked", GTK_SIGNAL_FUNC(callback_gtk_file_dialog_open),
                       (gpointer)func);

  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
                            "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), 
                              (gpointer)filesel);

  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button), 
                            "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
                              (gpointer)filesel);

  gtk_file_selection_complete(GTK_FILE_SELECTION(filesel), pattern);

  gtk_window_set_position(GTK_WINDOW(filesel), GTK_WIN_POS_CENTER);

  gtk_window_set_transient_for(GTK_WINDOW(filesel), GTK_WINDOW(parent));

  gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);

  gtk_widget_show_now(filesel);
  gdk_window_raise(filesel->window);
}
void callback_gtk_file_dialog_open(GtkWidget *widget, gpointer gd)
{
  ((void (*)(char *))gd)((char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
}



/*--[ Confirmation Dialog ]---------------------------------------------------*/
void gtk_confirmation_dialog_open(GtkWidget *parent, const char *title,
                                  const char *message, void (*func)(void))
{
  GtkWidget *dialog = gtk_dialog_new();
  GtkWidget *btn_ok = gtk_button_new_with_label("Yes");
  GtkWidget *btn_ng = gtk_button_new_with_label("No");
  GtkWidget *lb_msg = gtk_label_new(message);

  gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
  gtk_label_set_justify(GTK_LABEL(lb_msg), GTK_JUSTIFY_LEFT);

  gtk_window_set_title(GTK_WINDOW(dialog), title);
  gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(parent));
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

  gtk_signal_connect(GTK_OBJECT(btn_ok), "clicked",
                     GTK_SIGNAL_FUNC(callback_gtk_confirmation_dialog_open), (gpointer)func);
  gtk_signal_connect_object(GTK_OBJECT(btn_ok), "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
  gtk_signal_connect_object(GTK_OBJECT(btn_ng), "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), lb_msg);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 
                    (gpointer)btn_ok);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 
                    (gpointer)btn_ng);

  gtk_widget_show(btn_ok);
  gtk_widget_show(btn_ng);
  gtk_widget_show(lb_msg);
  gtk_widget_show_now(dialog);
  gdk_window_raise(dialog->window);
  gtk_widget_grab_focus(btn_ng);
}
void callback_gtk_confirmation_dialog_open(GtkWidget *widget, gpointer gd)
{
  ((void (*)(void))gd)();
}



/*--[ Message Dialog ]--------------------------------------------------------*/
void gtk_message_dialog_open(GtkWidget *parent, 
                             const char *title, const char *message)
{
  GtkRequisition req;
  GtkWidget *dialog = gtk_dialog_new();
  GtkWidget *btn_ok = gtk_button_new_with_label("OK");
  GtkWidget *lb_help = gtk_label_new(message);

  gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
  gtk_label_set_justify(GTK_LABEL(lb_help), GTK_JUSTIFY_LEFT);
  gtk_window_set_title(GTK_WINDOW(dialog), title);
  gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(parent));
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

  gtk_signal_connect_object(GTK_OBJECT(btn_ok), "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)dialog);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), lb_help);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), btn_ok,
                     FALSE, FALSE, 0);
  gtk_widget_size_request(btn_ok, &req);
  req.width += 64;
  gtk_widget_set_usize(btn_ok, req.width, req.height);

  gtk_widget_show(btn_ok);
  gtk_widget_show(lb_help);
  gtk_widget_show_now(dialog);
  gdk_window_raise(dialog->window);
  gtk_widget_grab_focus(btn_ok);
}



/*--[ fput_? Rouchines ]------------------------------------------------------*/
void fput_l(const unsigned long number, FILE *fp)
{
  fputc(number & 0xFF, fp);
  fputc((number >> 8) & 0xFF, fp);
  fputc((number >>16) & 0xFF, fp);
  fputc((number >>24) & 0xFF, fp);
}
void fput_s(const unsigned short number, FILE *fp)
{
  fputc(number & 0xFF, fp);
  fputc(number >> 8, fp);
}
void fput_str(const unsigned char *string, const int size, FILE *fp)
{
  int i;
    for(i=0; i<size; i++)
    {
      fputc(string[i], fp);
    }
}



/*--[ Check Extention ]-------------------------------------------------------*/
int chk_ext(const char *name, const char *ext)
{
  return strcmp_ci(name+strlen(name)-strlen(ext), ext);
}
int strcmp_ci(const char *str1, const char *str2)
{
    while(tolower(*str1) == tolower(*str2))
    {
        if(*str1 == '\0')
        {
          return 0;
        }
      str1++;
      str2++;
    }
  return tolower(*str1) - tolower(*str2);
}



/*--[ Structures of Widgets ]---------------------------------------------------
GtkWindow - window
	GtkVBox - vb_menu
		GtkMenuBar - menu
		GtkHBox - hb_main
			GtkEventBox - eb_main
				GtkDrawingArea - da_main
			GtkVBox - vb_sub
				GtkLabel - lb_desc
				GtkEventBox - eb_sub
					GtkDrawingArea - da_sub
*/

